package com.franckbarbier.From_Java_9;

import static com.franckbarbier.From_Java_9.Direction.North;

enum Direction {
    East, South, West, North
}

public class From_Java_9 {

    public static void main(String[] args) {
        // From Java 9:
        Mammutidae m0 = new Mammutidae("Macron");
        Mammutidae m1 = new Mammutidae("Poutine");
        Mammutidae m2 = new Mammutidae("Trump");
        System.out.println(m1.description()); // Inconsistency comes from failing overriding...

        // Double braces:
        java.util.Set<Mammutidae> mammutidae = new java.util.HashSet<>() { // <- Creation of an anonymous inner class, which extends 'java.util.HashSet'
            { // <- Block initializer
                add(new Mammutidae("Macron"));
                add(new Mammutidae("Poutine"));
                add(new Mammutidae("Trump"));
            }
        };

        // Immutable collections from Java 9 (e.g., a set):
        try {
            java.util.Set.of(m1, m2).add(m0); // Poutine and Trump don't want Macron!
        } catch (UnsupportedOperationException uoe) {
            System.out.println("In essence, immutable collections do not support changes like additions: " + uoe.toString());
        }

        // From Java 10:
        var polynomial = new java.util.Hashtable<Double, Double>();
        polynomial.put(1., -12.);
        polynomial.put(39., 8.);
        // From Java 12:
        var my_direction = Direction.East;
        var next_direction = switch (my_direction) { // https://mkyong.com/java/java-how-to-enable-the-preview-language-features/
            case East ->
                /* 'break' disappears! */ Direction.South;
            case South
                , North ->
                /* 'break' disappears and multiple choices! */ Direction.East;
            default ->
                my_direction;
        };
    }
}
